"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var node_fetch_1 = require("node-fetch");
var querystring = require("querystring");
var QiitaError_1 = require("../errors/QiitaError");
var QiitaForbiddenError_1 = require("../errors/QiitaForbiddenError");
var QiitaInternalServerError_1 = require("../errors/QiitaInternalServerError");
var QiitaNotFoundError_1 = require("../errors/QiitaNotFoundError");
var QiitaRateLimitError_1 = require("../errors/QiitaRateLimitError");
var QiitaUnauthorizedError_1 = require("../errors/QiitaUnauthorizedError");
var QiitaURLResolveError_1 = require("../errors/QiitaURLResolveError");
var Gateway = /** @class */ (function () {
    /**
     * @param options オプショナルなパラメーター
     * @param options.url Qiitaのホストです
     * @param options.version APIバージョンを示すロケーションです
     * @param options.token ユーザーのアクセストークンです
     */
    function Gateway(options) {
        /** Qiitaのホストです */
        this.url = 'https://qiita.com';
        /** ユーザーのアクセストークンです */
        this.token = '';
        /** APIバージョンを示すロケーションです */
        this.version = '/api/v2';
        if (options && options.url) {
            this.url = options.url;
        }
        if (options && options.version) {
            this.version = options.version;
        }
        if (options && options.token) {
            this.token = options.token;
        }
    }
    /**
     * Qiita APIにアクセスするためのトークンを設定します
     * @param token トークン文字列
     * @return 何も返しません
     */
    Gateway.prototype.setToken = function (token) {
        this.token = token;
    };
    /**
     * Qiita Teamへのエンドポイントを設定します
     * @param url Qiitaのホスト
     * @return 何も返しません
     */
    Gateway.prototype.setUrl = function (url) {
        this.url = url;
    };
    /**
     * Qiita APIへのパスを指定します．
     * @param version APIへのパスの文字列 (e.g. `/api/v2`)
     * @return 何も返しません
     */
    Gateway.prototype.setVersion = function (version) {
        this.version = version;
    };
    /**
     * Fetch APIのラッパー関数です
     * @param url リクエストするURLです
     * @param options Fetch APIの第二引数に渡されるオプションです
     * @return パースされたJSONオブジェクトを解決するPromiseです
     */
    Gateway.prototype.request = function (url, options) {
        if (options === void 0) { options = {}; }
        return __awaiter(this, void 0, void 0, function () {
            var response, _a, data;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        if (!options.headers) {
                            options.headers = {};
                        }
                        if (!options.headers['Content-Type']) {
                            options.headers['Content-Type'] = 'application/json';
                        }
                        if (!this.url) {
                            throw new QiitaURLResolveError_1.QiitaURLResolveError('Qiitaのホストが指定されていません。`Qiita.setUrl` でAPIのホストを指定してからメソッドを呼び出してください。');
                        }
                        if (this.token) {
                            options.headers.Authorization = "Bearer " + this.token;
                        }
                        if (!(typeof window === 'undefined')) return [3 /*break*/, 2];
                        return [4 /*yield*/, node_fetch_1.default(url, options)];
                    case 1:
                        _a = _b.sent();
                        return [3 /*break*/, 4];
                    case 2: return [4 /*yield*/, fetch(url, options)];
                    case 3:
                        _a = _b.sent();
                        _b.label = 4;
                    case 4:
                        response = _a;
                        return [4 /*yield*/, response.json()];
                    case 5:
                        data = _b.sent();
                        if (response.ok) {
                            return [2 /*return*/, data];
                        }
                        else {
                            // Qiitaがエラーの際に返すステータスコードが明記されていなかったので
                            // ありそうなものをハンドルしています。
                            // ref: https://qiita.com/api/v2/docs#%E3%82%B9%E3%83%86%E3%83%BC%E3%82%BF%E3%82%B9%E3%82%B3%E3%83%BC%E3%83%89
                            switch (response.status) {
                                case 401:
                                    throw new QiitaUnauthorizedError_1.QiitaUnauthorizedError(data.message || 'リクエストに必要な権限が不足しています。');
                                case 403:
                                    throw new QiitaForbiddenError_1.QiitaForbiddenError(data.message || 'このリクエストは禁止されています。');
                                case 404:
                                    throw new QiitaNotFoundError_1.QiitaNotFoundError(data.message || '指定したエンドポイントが見つかりませんでした');
                                case 429:
                                    throw new QiitaRateLimitError_1.QiitaRateLimitError(data.message || 'APIのレートリミットに到達しました。時間をおいてもう一度お試しください。');
                                case 500:
                                    throw new QiitaInternalServerError_1.QiitaInternalServerError(data.message || 'Qiitaのサーバーが internal server error を返しました。ホストが混雑している可能性がありますので、時間をおいてもう一度お試しください。');
                                default:
                                    throw new QiitaError_1.QiitaError('QiitaError', data.message || 'Qiita APIのリクエスト中に予期せぬエラーが発生しました');
                            }
                        }
                        return [2 /*return*/];
                }
            });
        });
    };
    /**
     * HTTP GETのラッパー関数です
     * @param url リクエストするURL
     * @param params クエリ文字列
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.get = function (url, params, options) {
        if (params === void 0) { params = {}; }
        if (options === void 0) { options = {}; }
        return this.request(url + (Object.keys(params).length ? '?' + querystring.stringify(params) : ''), __assign({ method: 'GET' }, options));
    };
    /**
     * HTTP POSTのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.post = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(url, __assign({ method: 'POST', body: JSON.stringify(body) }, options));
    };
    /**
     * HTTP PUTのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.put = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(url, __assign({ method: 'PUT', body: JSON.stringify(body) }, options));
    };
    /**
     * HTTP DELETEのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.delete = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(url, __assign({ method: 'DELETE', body: JSON.stringify(body) }, options));
    };
    /**
     * HTTP PATCHのラッパー関数です
     * @param url リクエストするURL
     * @param body リクエストボディ
     * @param options Fetch APIの第二引数になるオブジェクト
     */
    Gateway.prototype.patch = function (url, body, options) {
        if (body === void 0) { body = {}; }
        if (options === void 0) { options = {}; }
        return this.request(url, __assign({ method: 'PATCH', body: JSON.stringify(body) }, options));
    };
    return Gateway;
}());
exports.Gateway = Gateway;
//# sourceMappingURL=Gateway.js.map